﻿Imports System.IO

Namespace Roton

    Partial Public NotInheritable Class Environment

        Private FileCache As New List(Of CachedFile)

        Private Class CachedFile
            Public Data() As Byte
            Public Name As String

            Public Sub New(ByVal NewName As String, ByVal Source As Stream, ByVal Length As Integer)
                Dim Reader As New BinaryReader(Source)

                Name = NewName.ToUpper
                Data = Reader.ReadBytes(Length)
            End Sub
        End Class

        Public Sub AssociateFile(ByVal FileName As String, ByVal Source As Stream, ByVal Length As Integer)
            ' add a file to the cache, disallow duplicates
            FileName = My.Computer.FileSystem.GetName(FileName)
            UnassociateFile(FileName)
            FileCache.Add(New CachedFile(FileName, Source, Length))
        End Sub

        Public Sub UnassociateFile(ByVal FileName As String)
            Dim Index As Integer
            Dim Max As Integer = FileCache.Count - 1

            ' remove a file from the cache
            For Index = Max To 0 Step -1
                If (FileCache(Index).Name = FileName) Then
                    FileCache.RemoveAt(Index)
                End If
            Next
        End Sub

        Protected Function GetFile(ByVal FileName As String) As Byte()
            Dim Success As Boolean = False
            Dim Ret() As Byte = {}
            Dim Index As Integer
            Dim Count As Integer = FileCache.Count - 1
            Dim FileResult As Collections.ObjectModel.ReadOnlyCollection(Of String)

            ' search the cache first
            For Index = 0 To Count
                If (FileCache(Index).Name = FileName) Then
                    Ret = FileCache(Index).Data
                    Success = True
                    Exit For
                End If
            Next

            ' if we don't find the file in the cache, we search the disk if path exists
            If (Not Success) AndAlso (Path <> "") Then
                With My.Computer.FileSystem
                    FileResult = .GetFiles(Path)
                    For Index = 0 To FileResult.Count - 1
                        If (.GetName(FileResult(Index)).ToUpper = FileName) Then
                            Ret = .ReadAllBytes(FileResult(0))
                            Success = True
                            Exit For
                        End If
                    Next
                End With
            End If

            Return Ret
        End Function

        Protected Function GetFileStream(ByVal FileName As String) As MemoryStream
            Return New MemoryStream(GetFile(FileName))
        End Function

    End Class

End Namespace